exports.RingBuffer = function(maxSize) {
    if (maxSize <= 0) throw new Error('Cannot create RingBuffer with size ' + maxSize);
    var data = [];
    var cursor = 0;
    var isFull = false;

    this.append = function(x) {
        if (isFull) {
            data[cursor] = x;
            cursor = (cursor + 1) % maxSize;
        } else {
            data.push(x);
            cursor++;
            if (data.length === maxSize) {
                cursor = 0;
                isFull = true;
            }
        }
    };

    this.avg = function() {
        if (cursor === 0) return 0;
        var sum = data.reduce(function(a, b) { return a + b });
        return sum / (isFull ? maxSize : cursor);
    };

    this.size = function() {
        return isFull ? maxSize : cursor;
    };

    this.clear = function() {
        data = [];
        cursor = 0;
        isFull = false;
    };
};