const Stratum = require('stratum-pool');

// Refer to the README.md file for information on each field in the below configuration.
// This pool is configured for Gewel.
const pool = Stratum.createPool({
    coin: {
        name: 'Gewel',
        symbol: 'GWL',
        algorithm: 'x11',
        nValue: 1024,
        rValue: 1,
        txMessages: false,
        peerMagic: 'aa1b69b0',
        peerMagicTestnet: 'efa2faf7'
    },
    auxes: [],
    // Address to where block rewards are given
    address: "Xxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxx",
    rewardRecipients: {
        // No fixed reward recipients at this time.
        // Perhaps will add pool operators in the future.
    },
    blockRefreshInterval: 1000,
    jobRebroadcastTimeout: 55,
    connectionTimeout: 600,
    emitInvalidBlockHashes: false,
    tcpProxyProtocol: false,
    banning: {
        enabled: false,
        time: 600,
        invalidPercent: 50,
        checkThreshold: 500,
        purgeInterval: 300
    },
    ports: {
        3032: {
            diff: 0.005,
            varDiff: {
                minDiff: 8,
                maxDiff: 512,
                targetTime: 15,
                retargetTime: 90,
                variancePercent: 30
            }
        },
        3256: {
            diff: 0.01
        }
    },
    daemons: [
        {
            host: '127.0.0.1',
            port: 19998,
            user: 'gewel',
            password: 'local321'
        }
    ],
    p2p: {
        enabled: false,
        host: '127.0.0.1',
        port: 19999,
        disableTransactions: true
    }
}, function(ip, port , workerName, password, callback) {
    console.log('Authorize ' + workerName + ':' + password + '@' + ip);
    callback({
        error: null,
        authorized: true,
        disconnect: false
    });
});

// Refer to the README.md file for details on what the specification of data is.
pool.on('share', function(isValidShare, isValidBlock, data) {
    if (isValidBlock) {
        console.log('Block found');
    } else if (isValidShare) {
        console.log('Valid share submitted');
    } else if (data.blockHash) {
        console.log('We thought a block was found but it was rejected by the daemon');
    } else {
        console.log('Invalid share submitted');
        console.log('share data: ' + JSON.stringify(data));
    }
});

pool.on('block', function(coin, height, blockHash, txHash) {
    console.log('Mined block on ' + coin + ' network!');
    console.log('HEIGHT: ' + height);
    console.log('HASH: ' + blockHash);
    console.log('TX: ' + txHash);
});

pool.on('log', function(severity, logKey, logText) {
    console.log(severity + ': ' + '[' + logKey + '] ' + logText);
});

pool.start();
